#include "StdAfx.h"
#include "WASGetTotalUsedCapa.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"

CWASGetTotalUsedCapa::CWASGetTotalUsedCapa(CWASActor *pActor) : CWASCommand(pActor)
{
}

CWASGetTotalUsedCapa::~CWASGetTotalUsedCapa(void)
{
}


void CWASGetTotalUsedCapa::ExecuteAsync()
{
	USES_CONVERSION;
	USES_UTF8_CONVERSION;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/totalUsedCapa.php");

	m_httpFacade.session().SetURL(T2CA(strURL.c_str()),	pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);
	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	
	m_httpFacade.StopRequest();
	m_httpFacade.SendRequest(this);
}


bool CWASGetTotalUsedCapa::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	m_sUsedCapa.Clear();

	USES_UTF8_CONVERSION;
	
	do
	{
		nfc::IIkXMLAttribute * pAttr;
		nfc::IIkXMLNode * pNode;
		pNode = pDom->GetdocumentElement()->selectSingleNode("capa");
		if(pNode == NULL)
			break;

		pAttr = pNode->getAttribute("limitedCapa");
		if(pAttr == NULL)
			break;
		m_sUsedCapa.nLimitedCapa = atoi(pAttr->Gettext().c_str());

		pAttr = pNode->getAttribute("totalUsedCapa");
		if(pAttr == NULL)
			break;
		m_sUsedCapa.nTotalUsedCapa = atoi(pAttr->Gettext().c_str());

	} while(false);

	

	return true;
}


// override in drived class
void CWASGetTotalUsedCapa::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	/////////////////////////////////
	// 뷮 
	GT_GET_TOTAL_USED_CAPA_DATA sData;
	sData.nLimitedCapa = m_sUsedCapa.nLimitedCapa;
	sData.nTotalUsedCapa = m_sUsedCapa.nTotalUsedCapa;	

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetTotalUsedCapa(m_pActor, GetID(), m_eError, sData) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASGetTotalUsedCapa::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	/////////////////////////////////
	// 뷮 
	GT_GET_TOTAL_USED_CAPA_DATA sData;
	sData.nLimitedCapa = m_sUsedCapa.nLimitedCapa;
	sData.nTotalUsedCapa = m_sUsedCapa.nTotalUsedCapa;	

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetTotalUsedCapa(m_pActor, GetID(), m_eError, sData) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}